/*
 * FILE --- rls_estm.hh
 *
 * PURPOSE --- Header file for a class that implements a Recursive 
 * Least Squares (RLS) algorithm for parameter estimation
 * for Single-Input-Single-Output (SISO) plants.
 * The user specifies the dimension of the parameter vector, the 
 * initial parameter vector and initial covariance matrix.
 * Moreover, the user can set a forgetting factor lambda, such that
 * 0 < lambda <= 1, where lambda = 1 is the no forgetting case.
 * 
 * This utility uses the matrix tool for matrix maltiplications.
 *
 */

// Prevent Multiple Inclusion
#ifndef RLS_ESTM_HH
#define RLS_ESTM_HH

// Include Files
#include <stdlib.h>
#include "matrix.hh"

// Local Definitions
#define RLS_ESTM_NO_ERROR 0
#define RLS_ESTM_MEM_ERROR 1

class RLS_ESTM
{
public:
  
  // Constructor
  RLS_ESTM(int _num_of_param, 
	   double *_theta_0 = NULL, 
	   double **_P_0 = NULL);
  
  // Destructor
  ~RLS_ESTM();
  
  // Set the or return forgetting factor lambda
  inline double get_lambda() const { return(lambda); }; 
  void set_lambda(double _lambda); 
  
  // Set the or return rls parameters
  void set_theta(const double *_theta); // Set theta to the specified value
  void get_theta(double *_theta); // Return the current value of theta
  
  // Set or return the rls covarinace matrix
  void set_P(const double **_P); // Set P to the specified value
  void set_P(double _diagonal_val); // Set P to identity times the given scalar
  void get_P(double **_P); // Return the current value of P
  
  // Calculate the new estimate 
  double estimate(const double plant_output, 
		  const double *regression_vector,
		  double *estimates);
  
  int rls_estm_err;  // Variable to hold the error condition
  
private:
  // # of parameters to be estimated
  int num_of_param;
  
  // The parameter vector and the covariance matrix
  matrix *theta;
  matrix *P;
  
  // Regression vector (conveniance variable)
  matrix *h;
  
  // Forgetting factor
  double lambda;
  
  // Convaniance variables
  double output_error;
  double temp;
  matrix *Ph;
};

#endif // RLS_ESTM_HH
